disp(' ');
disp(' beam_base_accel_fea.m  ver 1.6  January 14, 2014 ');
disp(' ');
disp(' by Tom Irvine ');
disp(' Email: tom@vibrationdata.com ');
disp(' ');
disp(' This script calculates the natural frequencies of a beam ');
disp(' via the finite element method. It also calculates the    ');
disp(' frequency response functions for base excitation.        ');
disp(' ');
%
close all hidden;
%
clear stiff;
clear mass;
%
clear Mdd;  % driven
clear Mdf;
clear Mfd;
clear Mff;  % free
%
clear Mdw;
clear Mwd;
clear Mww;
%
clear MP;
clear MT;
%
clear Kdd;  
clear Kdf;
clear Kfd;
clear Kff;
%
clear invKff;
%
clear Kww;
%
clear KP;
clear KT;
%
clear n;
clear na;
clear nv;
clear nd;
%
clear m;
clear k;
%
clear TT;
clear T1;
clear T2;
%
clear I;
clear Idd;
clear Iff;
%
clear damp;
clear ea;
clear f;
clear ngw;
clear C;
clear freq;
clear nodex;
clear dof;
clear nem;
%
clear omega;
clear omegan;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
fig_num=1;
%
disp(' Enter units:  1=English 2=metric ');
iu=input(' ');
%
[E,I,A,rho,ipin,ne,total_length,xv,dx,dof,ipm,pm,xpm,nodex]=...
                                                   beam_data_entry_base(iu);
%
%
dof_orig=2*ne+2;
%
[klocal] = local_stiffness(E,I,ne,dx);
%
[mlocal] = local_mass(rho,ne,dx);
%
[stiff,mass,dof_status]=...
                    assembly_base(ne,klocal,mlocal,ipin,dof,ipm,pm,xpm,dx);
%
sz=size(mass);
dof=(sz(1));
num=dof;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
out1=sprintf('\n number of dofs =%d \n',num);
disp(out1);
%
disp(' ');
disp(' Select the base excitation drive points ');
disp('  1=left only ');
disp('  2=right only ');
disp('  3=both ');
%
iap=input(' ');
%
%%
%%%% disp('  Constraint rotational dof ');
%%%% disp('  1=none '); 
%%%% disp('  2=left end only ');
%%%% disp('  3=left & right ends  ')
%
if(iap==1) % left only enforced
    ea(1)=1;
    nem=1;
end
%
if(iap==2) % right only enforced
%    
    if(ipin==1) % none
        ea(1)= num-1;
    end
    if(ipin==2) % left only constrained
        ea(1)= num-1;
    end
    if(ipin==3) % left & right constrained
        ea(1)= num; 
    end
    nem=1;
end
%
if(iap==3) % both enforced
%    
    ea(1)=1;
%    
    if(ipin==1) % none
        ea(2)= num-1;
    end
    if(ipin==2) % left only constrained
        ea(2)= num-1;      
    end
    if(ipin==3) % left & right constrained
        ea(2)= num;       
    end
    nem=2;
end
%
disp(' ');
%
[nff,ngw,Mwd,Mww,fn,omegan,ModeShapes,MST,part,T1,T2]=...
 partition_matrices_accel(num,nem,ea,mass,stiff);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
[damp]=enter_modal_damping(num);
%
[acc_frf,fstart,fend,freq]=...
 partition_matrices_accel_frf(num,nff,nem,ngw,Mwd,Mww,fn,damp,omegan,...
                                          ModeShapes,MST,part,T1,T2);
%
clear length;
disp(' ');
if(iu==1)
    disp(' node  position(inch) ');
else
    disp(' node  position(meters) ');    
end     
for i=1:length(nodex)
    out1=sprintf(' %d  %8.4g',i,nodex(i));
    disp(out1);
end    
%
[trans_acc_frf,trans_vel_frf,trans_disp_frf,trans_rel_disp_frf,fig_num]=...
    beam_base_fea_post(ipin,nodex,acc_frf,freq,fstart,fend,fig_num,iu);
%
%
PF=part;
%
disp(' ');
disp(' Apply half-sine base input?  1=yes  2=no ');
isb=input(' ');
if(isb==1)
  [t,acc,vel,rel_disp,abase]=...
             mdof_half_sine_core(fn,damp,PF,ModeShapes,T1,T2,ngw,iu);
  [acc_half_sine,rel_disp_half_sine,fig_num]=...
             beam_base_th_plot(t,acc,vel,rel_disp,abase,nodex,ipin,fig_num,iu);
end
%
disp(' ');
disp(' Apply arbitrary time history base input?  1=yes  2=no ');
iar=input(' ');
if(iar==1)
%  
  [t,acc,vel,rel_disp,abase]=...
        mdof_arbit_base_core(fn,damp,PF,ModeShapes,T1,T2,ngw,iu);
  [acc_arb,rel_disp_arb,fig_num]=...
        beam_base_th_plot(t,acc,vel,rel_disp,abase,nodex,ipin,fig_num,iu);    
end
%
disp(' ');
disp(' Output arrays: ');
disp(' ');
disp('   trans_acc_frf');
disp('   trans_vel_frf');
disp('   trans_disp_frf');
disp('   trans_rel_disp_frf');
%
if(isb==1)
    disp('   acc_half_sine');
    disp('   rel_disp_half_sine');       
end
%
if(iar==1)
    disp('   acc_arb');
    disp('   rel_disp_arb');       
end